# Clip Color to Marker Converter for DaVinci Resolve
# by Daniel Bañuelos | More tools at dandbc.mx/tools

import sys
import os
import tkinter as tk
from tkinter import messagebox, filedialog
import csv

# Attempt to import DaVinciResolveScript using environment variables
try:
    from python_get_resolve import GetResolve
    resolve = GetResolve()
    if not resolve:
        raise ImportError("GetResolve() returned None")
    print("✅ Successfully connected to DaVinci Resolve using GetResolve().")
except ImportError:
    print("⚠️ Failed to import using GetResolve(). Attempting manual path setup.")
    # Fallback to manual import
    resolve_script_path = os.environ.get("RESOLVE_SCRIPT_API", "/Library/Application Support/Blackmagic Design/DaVinci Resolve/Developer/Scripting")
    sys.path.append(os.path.join(resolve_script_path, "Modules"))
    try:
        import DaVinciResolveScript as dvr_script
        resolve = dvr_script.scriptapp("Resolve")
        if not resolve:
            print("❌ Failed to connect to DaVinci Resolve API.")
            sys.exit()
        print("✅ Successfully connected to DaVinci Resolve using manual path.")
    except ImportError as e:
        print("❌ Error importing DaVinciResolveScript:", e)
        sys.exit()

# Initialize project and timeline
project_manager = resolve.GetProjectManager()
current_project = project_manager.GetCurrentProject()
if not current_project:
    print("❌ No project is loaded in DaVinci Resolve.")
    sys.exit()

timeline = current_project.GetCurrentTimeline()
if not timeline:
    print("❌ No active timeline found in DaVinci Resolve.")
    sys.exit()

# Updated Clip color mapping with DaVinci Resolve's official colors
clip_color_map = {
    "Orange": "#fd6400",
    "Apricot": "#ffa300",
    "Yellow": "#eca600",
    "Lime": "#94c700",
    "Olive": "#4a9b00",
    "Green": "#189161",
    "Teal": "#009b9a",
    "Navy": "#00547b",
    "Blue": "#2f78a5",
    "Purple": "#a070a3",
    "Violet": "#e14b8f",
    "Pink": "#f886b7",
    "Tan": "#bbaf94",
    "Beige": "#cb9e77",
    "Brown": "#a26300",
    "Chocolate": "#945739"
}

# Function to convert frames to timecode
def frames_to_timecode(frames, fps):
    hours = int(frames / (3600 * fps))
    minutes = int((frames % (3600 * fps)) / (60 * fps))
    seconds = int((frames % (60 * fps)) / fps)
    remaining_frames = int(frames % fps)
    return f"{hours:02}:{minutes:02}:{seconds:02}:{remaining_frames:02}"

# Function to convert clip colors to markers and export EDL
def convert_clip_colors_to_edl(selected_tracks, selected_colors_notes):
    track_count = timeline.GetTrackCount("video")
    edl_data = []

    # Get the project's frame rate
    fps = float(current_project.GetSetting("timelineFrameRate"))

    edl_data.append("TITLE: Clip Color Markers Export")
    edl_data.append("FCM: NON-DROP FRAME\n")

    event_num = 1

    for track_index in range(1, track_count + 1):
        if str(track_index) not in selected_tracks:
            continue

        clips = timeline.GetItemListInTrack("video", track_index)
        if not clips:
            print(f"⚠️ No clips found on track V{track_index}")
            continue

        for clip in clips:
            clip_name = clip.GetName()
            clip_color = clip.GetClipColor()
            start_frame = clip.GetStart()
            end_frame = clip.GetEnd()
            track_name = f"V{track_index}"

            if clip_color not in selected_colors_notes:
                continue

            # Convert start_frame and end_frame to timecode
            start_timecode = frames_to_timecode(start_frame, fps)
            end_timecode = frames_to_timecode(end_frame, fps)

            # Custom note for this color
            custom_note = selected_colors_notes[clip_color]

            # EDL Event
            edl_event = f"{event_num:003}  AX       V     C        {start_timecode} {end_timecode} {start_timecode} {end_timecode}"
            edl_data.append(edl_event)
            edl_data.append(f"* FROM CLIP NAME: {clip_name}")
            edl_data.append(f"* COMMENT: Color: {clip_color} -- Note: {custom_note}")
            edl_data.append("")
            event_num += 1

    # Export EDL
    file_path = filedialog.asksaveasfilename(defaultextension=".edl", filetypes=[("EDL files", "*.edl")])
    if not file_path:
        return

    with open(file_path, 'w') as edl_file:
        edl_file.write('\n'.join(edl_data))

    messagebox.showinfo("Success", "EDL exported successfully.")

# GUI for selecting tracks, colors, and adding notes
def launch_gui():
    root = tk.Tk()
    root.title("Clip Color to EDL Converter")

    tk.Label(root, text="Select Video Tracks:").pack()
    track_vars = {}
    track_count = timeline.GetTrackCount("video")
    for i in range(1, track_count + 1):
        var = tk.IntVar(value=1)  # Checked by default
        chk = tk.Checkbutton(root, text=f"V{i}", variable=var)
        chk.pack(anchor='w')
        track_vars[str(i)] = var

    tk.Label(root, text="Select Clip Colors and Add Notes:").pack()
    color_vars = {}
    note_entries = {}
    for color, hex_code in clip_color_map.items():
        frame = tk.Frame(root)
        frame.pack(anchor='w', fill='x')

        var = tk.IntVar(value=1)  # Checked by default
        chk = tk.Checkbutton(frame, variable=var)
        chk.grid(row=0, column=0)

        color_label = tk.Label(frame, text=color, bg=hex_code, fg="black" if hex_code != "#FFFFFF" else "black", width=10)
        color_label.grid(row=0, column=1)

        note_entry = tk.Entry(frame)
        note_entry.grid(row=0, column=2, padx=5)
        note_entry.insert(0, "")

        color_vars[color] = var
        note_entries[color] = note_entry

    def run_conversion():
        selected_tracks = [track for track, var in track_vars.items() if var.get() == 1]
        selected_colors_notes = {color: note_entries[color].get() for color, var in color_vars.items() if var.get() == 1}

        if not selected_tracks or not selected_colors_notes:
            messagebox.showwarning("Input Error", "Please select at least one track and one color.")
            return

        convert_clip_colors_to_edl(selected_tracks, selected_colors_notes)

    tk.Button(root, text="Convert Clip Colors to EDL", command=run_conversion).pack(pady=10)
    root.mainloop()

# Launch the GUI
if __name__ == "__main__":
    launch_gui()
